a:focus {
    outline: none;
}

.no-left-gutter {
    padding-left:0px;
}

.no-right-gutter {
    padding-right:0px;
}

.form-inline .form-group {
    margin-left:0;
    margin-right:0;
}

.navbar-right {
    margin-right:10px !important;
}

input.kv-key {
    width: 60% !important;
}
input.kv-value {
    width: 80% !important;
}

.container {
    margin-bottom:20px;
}

.choices-well {
    margin-bottom:2px;
}

.key-remove {
    width:auto;
}

.move-kv-down {
    margin-right:15px;
}

.kv-actions {
    padding-top:5px;
}

.add-choice-row {
    margin-top:15px;
}

/* Rating Field */
.rating {
    float:left;
    width:auto;
}

.rating:not(:checked) > input {
    position:absolute;
    top:-9999px;
    clip:rect(0,0,0,0);
}

.rating:not(:checked) > label {
    float:right;
    width:1.1em;
    padding:0 .1em;
    overflow:hidden;
    white-space:nowrap;
    cursor:pointer;
    font-size:200%;
    line-height:1.2;
    color:#ddd,
    text-shadow:1px 1px #bbb, 2px 2px #666, .1em .1em .2em rgba(0,0,0,0.5);
}

.rating:not(:checked) > label:before {
    content: '★ ';
}

.rating > input:checked ~ label {
    color: #f70;
    text-shadow:1px 1px #c60, 2px 2px #940, .1em .1em .2em rgba(0,0,0,.5);
}

.rating:not(:checked) > label:hover,
.rating:not(:checked) > label:hover ~ label {
    color: gold;
    text-shadow:1px 1px goldenrod, 2px 2px #B57340, .1em .1em .2em rgba(0,0,0,.5);
}

.rating > input:checked + label:hover,
.rating > input:checked + label:hover ~ label,
.rating > input:checked ~ label:hover,
.rating > input:checked ~ label:hover ~ label,
.rating > label:hover ~ input:checked ~ label {
    color: #ea0;
    text-shadow:1px 1px goldenrod, 2px 2px #B57340, .1em .1em .2em rgba(0,0,0,.5);
}

.form-group.required .control-label:after {
    color: red;
    content: "*";
    position: relative;
    margin-left: 5px;
    top: -2px;
    font-family: 'Glyphicons Halflings';
    font-weight: normal;
    font-size: 10px;
}

.required-mark {
    color: red;
    font-family: 'Glyphicons Halflings';
    font-weight: normal;
    font-size: 10px;
}

.required-info {
    font-size:120%;
    padding-top:5px;
    padding-bottom:10px;
}
