{% load staticfiles %}
<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="utf-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <title>{% block title %}{% endblock title %}</title>

  <link rel="stylesheet" 
  href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/css/bootstrap.min.css">
  <link rel="stylesheet" type="text/css"
    href="{% static 'dform/css/dform.css' %}"/>

  {% block extra_head %}
  {% endblock extra_head %}
</head>
<body>
  {% block nav %}
  {% endblock nav %}

  <div class="container">
  {% block contents %}
  {% endblock contents %}
  </div>

  <script 
    src="https://ajax.googleapis.com/ajax/libs/jquery/1.11.3/jquery.min.js">
  </script>
  <script 
    src="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/js/bootstrap.min.js">
  </script>
  {% block script %}
  {% endblock script %}
</body>
</html>
